#! /usr/bin/env python

#
# Python program for generating an index containing a link list
# of all html files in the specified directory, and corresponding index
# files for all subdirectories if this is wished. It takes the
# file's "title" and "description" meta information to print the
# file info.
#
# The constructed index files have the given name (or "dir_index.html"
# if nothing was specified as parameter), and are in the same directory
# as the files they describe. Links to the parent and child directories
# are provided for easy navigation.
#
# The program can be started with a number of parameters; type
# "make_index.py --help" for more info about this.
#
# IMPORTANT NOTE: I don't know if the program works on Windows systems
# since I did not check if all pathnames used are os-independently specified.
#
# Author: Chantal Keller
# Beta Release October 2000 (0.2)
#
# Pending:
# - program parameters for:
#   * subdirectories to exclude from scanning
#   * prob. location of stylesheet to use for visual presentation
# - extract more information from html files, e.g. first two lines
#   from first paragraph of text, and display them, too
#

import os
import os.path
import re
import string
import sys


### variables ###

depth = ''

usage = ("\n   " + sys.argv[0][2:] + ":\n\
   This program generates an index containing a link list of all html\n\
   files in current directory.\n\
   usage: make_index.py [-h|--help] [-v|--verbose] [-r|--recursive]\n\
   [-d<directory_path>] [-n<index_file_name>]\n\
   \n\
   Starting the program with the help flag displays this notice and then\n\
   exits.\n\
   \n\
   Running the program in verbose mode will list processig information.\n\
   \n\
   The recursive flag determines whether subdirectories of the specified\n\
   directory should be processed too. In this case, a tree of indices is\n\
   generated.\n\
   \n\
   If no directory containing the html files is specified, the program\n\
   will work on the current working directory.\n\
   \n\
   The name flag provides the name of the index file(s) to be generated.\n\
   If no name is given, the default name \"dir_index.html\" will be used.\n")

params_dic = {'dir':os.curdir,
              'verbose':0,
              'recursive':0,
              'name':"dir_index.html"}


### functions ###

# for verbose processing information
def debprint(string):
    if (verbose):
        print(string)

        
# generates the index file for the given directory 'dir', and
# for its subdirectories if 'rec' is positive;
# 'depth' is a string used for formatting
# 'navup' determines whether a navigation link to the
# parent directory should be provided
def generate_index_for_dir(dir, rec, depth, navup):

    os.chdir(os.getcwd()+"/"+dir)
    currentdir = os.getcwd()
    
    # collect all html files in specified directory
    debprint(depth+"Scanning directory "+currentdir+":")
    file_list = os.listdir(currentdir)
    file_list.sort()
    html_repr = re.compile(".*\.html?(?!~)")
    html_files = []
    dirs = []

    for file in file_list:
        if html_repr.match(file):
            # don't regard the index file itself
            if not re.match(index_file_name, file):
                html_files.append(file)
        elif os.path.isdir(file):
            dirs.append(file)
            
    # prepare index file 
    index = open(index_file_name, 'w')
    index.write("<HTML>\n\
   <HEAD>\n\
       <TITLE>Index for directory "+currentdir+"</TITLE>\n\
   <HEAD>\n\
   <BODY BGCOLOR=#FFFFFF VLINK=#000000 ALINK=#660000 LINK=#000000>\n\
      <FONT FACE=\"helvetica, arial\" SIZE=-1><CENTER>\n\
      <A NAME=\"top\"><P>&nbsp;</P></A>\n\
      <H2>Index of html files in directory "+os.getcwd()+":</H1>\n\
      <A HREF=\"#bottom\">Bottom</A>")
    
    if ((recursive) and (navup)):
        index.write("<P><A HREF=\"../"+index_file_name+"\">Go up to parent directory</P>\n")
        
    index.write("<HR SIZE=0 WIDTH=80%><P>")
    
    # for each html file, extract title and description information
    # and write it to index file (as a separate table)
    title_re = re.compile("<TITLE>", re.IGNORECASE)
    title_end_re = re.compile("</TITLE>", re.IGNORECASE)
    descr_re = re.compile("<META NAME=\"?DESCRIPTION\"? CONTENT=\"?", re.IGNORECASE)
    depth = depth+"---"
    for file in html_files:
        debprint(depth+"Scanning file "+file+"...")
        current_file = open("./"+file, 'r')
        line = current_file.readline()    
        title = ""
        descr = ""
        while line != "":
            match = title_re.search(line)
            if match:
                title = line[match.start()+7:]
                # check case of title tag alone in the line: go to next line
                if not re.search("[a-z,A-Z]", title):
                    line = current_file.readline()
                    title = line
                # check for title end tag and remove it
                if title_end_re.search(title):
                    title = re.split(title_end_re, title)[0]
                title = string.strip(title)
                
            match = descr_re.search(line)
            if match:
                descr = line[match.start()+34:]
                while not re.search(">", line):
                    line = current_file.readline()
                    descr = descr+line
                descr = re.split("\"?>", descr)[0] 
                descr = string.strip(descr)
                 
            # if we have left header, stop search
            if re.search("</HEAD>", line):
                line = ""
                
            line = current_file.readline()
         
        index.write("<TABLE BORDER=0 WIDTH=70%><TR><TD BGCOLOR=#E6E6FA><A HREF=\""+file+"\">"
                +title+"</A> <A HREF=\""+file+"\"><FONT COLOR=#777777> ("+file+")</FONT>\n\
                </A>:</TD></TR>\n\
                <TR><TD><BLOCKQUOTE>"+descr+"</BLOCKQUOTE></TD></TR></TABLE>\n")
    
    
    index.write("</UL>\n")
      

    # process subdirectories recursively if so specified
    if (recursive):

        # upward navigation shall be provides
        navup = 1

        if (not dirs == []):
            index.write("<P><HR SIZE=0 WIDTH=80%></P>\n")
        
        for subdir in dirs:
            # link to subdir's index file
            index.write("<P><A HREF=\""+subdir+"/"+index_file_name+"\">Index for \n\
            subdirectory <FONT COLOR=#6633CC>"+subdir+"</FONT></A>")

            # now start recursion...
            generate_index_for_dir(subdir, rec, depth, navup)
            os.chdir('..')
            

    # conclude and close index file
    index.write("<P><HR SIZE=0 WIDTH=80%></P>\n\
      <A HREF=\"#top\" NAME=\"bottom\">Top</A></P>\n\
      <EM><P>This file was automatically generated using <B>make_index</B>, a Python program.<BR>\n\
      Author: Chantal Keller</P></EM>\n\
      </CENTER></FONT>\n\
  </BODY>\n\
</HTML>")
    index.close() 
### main program ###

# first, check the parameter list
params = sys.argv[1:]
undef = 0

if (params == []):
    # no parameters given
    undef = 1
else:
    for param in params:
        if re.search("-d", param):
            params_dic['dir'] = param[2:]
        elif re.search("-v|--verbose", param):
            params_dic['verbose'] = 1
        elif re.search("-h|--help", param):
            print(usage)
            sys.exit(1)
        elif re.search("-r|--recursive", param):
            params_dic['recursive'] = 1
        elif re.search("-n", param):
            # check if name is "index.htm(l)" -> this is too
            # dangerous, so refuse to usse it
            name = param[2:]
            if (re.match("index.html?", name)):
                print("Choosing "+name+" as directory index file name is too dangerous, since\n\
this name is often used! Please choose another name.\nProgram aborted.")
                sys.exit(1)    
            params_dic['name'] = name
        else:
            # the parameter is not valid
            undef = 1

if (undef):
    # no parameters were given, or they didn't match;
    # check if we should proceed
    print("No valid parameters were specified. Should I proceed to generate an\n\
index file for the current directory, using the default settings? (y/n)")
    answer = ""
    while ((answer == "") or (not re.match("y|n", answer))):
        answer = sys.stdin.readline()
    if (re.match("n",answer)):
        print("Program aborted.")
        sys.exit(1)
    else:
        print("Proceeding...")
            
        
wdir = params_dic['dir']
os.chdir(wdir)
verbose = params_dic['verbose']
recursive = params_dic['recursive']
index_file_name = params_dic['name']

# now start generating
generate_index_for_dir('', recursive, "", 0)


        
